/// <reference types="node" />
import WS from "ws";
import { EventEmitter } from "events";
import _Telnet from "./telnet";
declare class WST extends EventEmitter {
    readonly UUID: string;
    private socket;
    readonly options: _Telnet.TelnetOptionMatrix;
    private responder;
    constructor(socket: WS);
    close(code?: number, reason?: string): void;
    send(data: Buffer): void;
    send(data: string): void;
    private sendTelnet;
    sendSubnegotiation(option: _Telnet.TelnetOption, data: Buffer): void;
    sendSubnegotiation(option: _Telnet.TelnetOption, data: string): void;
    sendGMCP(namespace: string, data: string): void;
    sendGMCP(namespace: string, data: {
        [key: string]: any;
    }): void;
    will(option: _Telnet.TelnetOption): void;
    wont(option: _Telnet.TelnetOption): void;
    do(option: _Telnet.TelnetOption): void;
    dont(option: _Telnet.TelnetOption): void;
    prompt(message: string, mask?: boolean): Promise<string>;
    private HandleData;
    on(event: "error", listener: (error: Error) => void): this;
    on(event: "message", listener: (data: string) => void): this;
    on(event: "close", listener: (code?: number, reason?: string) => void): this;
    on(event: "ping" | "pong", listener: (data?: Buffer) => void): this;
    on(event: "will" | "wont" | "do" | "dont", listener: (option: number) => void): this;
    on(event: "subnegotiation", listener: (option: number, data: Buffer) => void): this;
    on(event: "telnet", listener: (ev: _Telnet.TelnetEvent) => void): this;
    on(event: "open", listener: () => void): this;
    on(event: "gmcp", listener: (namespace: string, data: {
        [key: string]: any;
    }) => void): this;
    emit(event: "error", error: Error): boolean;
    emit(event: "message", data: string): boolean;
    emit(event: "close", code?: number, reason?: string): boolean;
    emit(event: "ping" | "pong", data?: Buffer): boolean;
    emit(event: "will" | "wont" | "do" | "dont", option: number): boolean;
    emit(event: "subnegotiation", option: number, data: Buffer): boolean;
    emit(event: "telnet", ev: _Telnet.TelnetEvent): boolean;
    emit(event: "open"): boolean;
    emit(event: "gmcp", namespace: string, data: {
        [key: string]: any;
    }): boolean;
}
declare namespace WST {
    interface WSSOptions {
        cert: string;
        key: string;
    }
    export class Server extends EventEmitter {
        readonly clients: Map<string, WST>;
        private _server;
        constructor(host: string, port: number, secure?: WSSOptions);
        on(event: "listening", listener: (host: string, port: number) => void): this;
        on(event: "close", listener: () => void): this;
        on(event: "error", listener: (error: Error) => void): this;
        on(event: "connection", listener: (client: WST) => void): this;
        emit(event: "listening", host: string, port: number): boolean;
        emit(event: "close"): boolean;
        emit(event: "error", error: Error): boolean;
        emit(event: "connection", client: WST): boolean;
    }
    export const Telnet: typeof _Telnet;
    export {};
}
export = WST;
