"use strict";
var Telnet;
(function (Telnet) {
    let TelnetNegotiation;
    (function (TelnetNegotiation) {
        /** Mark the start of a negotiation sequence. */
        TelnetNegotiation[TelnetNegotiation["IAC"] = 255] = "IAC";
        /** Confirm  */
        TelnetNegotiation[TelnetNegotiation["WILL"] = 251] = "WILL";
        /** Tell the other side that we refuse to use an option. */
        TelnetNegotiation[TelnetNegotiation["WONT"] = 252] = "WONT";
        /** Request that the other side begin using an option. */
        TelnetNegotiation[TelnetNegotiation["DO"] = 253] = "DO";
        /**  */
        TelnetNegotiation[TelnetNegotiation["DONT"] = 254] = "DONT";
        TelnetNegotiation[TelnetNegotiation["NOP"] = 241] = "NOP";
        /** Subnegotiation used for sending out-of-band data. */
        TelnetNegotiation[TelnetNegotiation["SB"] = 250] = "SB";
        /** Marks the end of a subnegotiation sequence. */
        TelnetNegotiation[TelnetNegotiation["SE"] = 240] = "SE";
        TelnetNegotiation[TelnetNegotiation["IS"] = 0] = "IS";
        TelnetNegotiation[TelnetNegotiation["SEND"] = 1] = "SEND";
    })(TelnetNegotiation = Telnet.TelnetNegotiation || (Telnet.TelnetNegotiation = {}));
    let TelnetOption;
    (function (TelnetOption) {
        /** Whether the other side should interpret data as 8-bit characters instead of standard NVT ASCII.  */
        TelnetOption[TelnetOption["BINARY_TRANSMISSION"] = 0] = "BINARY_TRANSMISSION";
        /** Whether the other side should continue to echo characters. */
        TelnetOption[TelnetOption["ECHO"] = 1] = "ECHO";
        TelnetOption[TelnetOption["RECONNECTION"] = 2] = "RECONNECTION";
        TelnetOption[TelnetOption["SUPPRESS_GO_AHEAD"] = 3] = "SUPPRESS_GO_AHEAD";
        TelnetOption[TelnetOption["APPROX_MESSAGE_SIZE_NEGOTIATION"] = 4] = "APPROX_MESSAGE_SIZE_NEGOTIATION";
        TelnetOption[TelnetOption["STATUS"] = 5] = "STATUS";
        TelnetOption[TelnetOption["TIMING_MARK"] = 6] = "TIMING_MARK";
        TelnetOption[TelnetOption["REMOTE_CONTROLLED_TRANS_ECHO"] = 7] = "REMOTE_CONTROLLED_TRANS_ECHO";
        TelnetOption[TelnetOption["OUTPUT_LINE_WIDTH"] = 8] = "OUTPUT_LINE_WIDTH";
        TelnetOption[TelnetOption["OUTPUT_PAGE_SIZE"] = 9] = "OUTPUT_PAGE_SIZE";
        TelnetOption[TelnetOption["OUTPUT_CR_DISPOSITION"] = 10] = "OUTPUT_CR_DISPOSITION";
        TelnetOption[TelnetOption["OUTPUT_HORIZONTAL_TAB_STOPS"] = 11] = "OUTPUT_HORIZONTAL_TAB_STOPS";
        TelnetOption[TelnetOption["OUTPUT_HORIZONTAL_TAB_DISPOSITION"] = 12] = "OUTPUT_HORIZONTAL_TAB_DISPOSITION";
        TelnetOption[TelnetOption["OUTPUT_FORMFEED_DISPOSITION"] = 13] = "OUTPUT_FORMFEED_DISPOSITION";
        TelnetOption[TelnetOption["OUTPUT_VERTICAL_TAB_STOPS"] = 14] = "OUTPUT_VERTICAL_TAB_STOPS";
        TelnetOption[TelnetOption["OUTPUT_VERTICAL_TAB_DISPOSITION"] = 15] = "OUTPUT_VERTICAL_TAB_DISPOSITION";
        TelnetOption[TelnetOption["OUTPUT_LINEFEED_DISPOSITION"] = 16] = "OUTPUT_LINEFEED_DISPOSITION";
        TelnetOption[TelnetOption["EXTENDED_ASCII"] = 17] = "EXTENDED_ASCII";
        TelnetOption[TelnetOption["LOGOUT"] = 18] = "LOGOUT";
        TelnetOption[TelnetOption["BYTE_MACRO"] = 19] = "BYTE_MACRO";
        TelnetOption[TelnetOption["DATA_ENTRY_TERMINAL"] = 20] = "DATA_ENTRY_TERMINAL";
        TelnetOption[TelnetOption["SUPDUP"] = 21] = "SUPDUP";
        TelnetOption[TelnetOption["SUPDUP_OUTPUT"] = 22] = "SUPDUP_OUTPUT";
        TelnetOption[TelnetOption["SEND_LOCATION"] = 23] = "SEND_LOCATION";
        TelnetOption[TelnetOption["TERMINAL_TYPE"] = 24] = "TERMINAL_TYPE";
        TelnetOption[TelnetOption["END_OF_RECORD"] = 25] = "END_OF_RECORD";
        TelnetOption[TelnetOption["TACACS_USER_IDENTIFICATION"] = 26] = "TACACS_USER_IDENTIFICATION";
        TelnetOption[TelnetOption["OUTPUT_MARKING"] = 27] = "OUTPUT_MARKING";
        TelnetOption[TelnetOption["TERMINAL_LOCATION_NUMBER"] = 28] = "TERMINAL_LOCATION_NUMBER";
        TelnetOption[TelnetOption["TELNET_3270_REGIME"] = 29] = "TELNET_3270_REGIME";
        TelnetOption[TelnetOption["X3_PAD"] = 30] = "X3_PAD";
        /**
         * Whether to negotiate about window size (client).
         * @example
         * [IAC, SB, NAWS, WIDTH[1], WIDTH[0], HEIGHT[1], HEIGHT[0], IAC, SE]
         */
        TelnetOption[TelnetOption["NEGOTIATE_ABOUT_WINDOW_SIZE"] = 31] = "NEGOTIATE_ABOUT_WINDOW_SIZE";
        TelnetOption[TelnetOption["TERMINAL_SPEED"] = 32] = "TERMINAL_SPEED";
        TelnetOption[TelnetOption["REMOTE_FLOW_CONTROL"] = 33] = "REMOTE_FLOW_CONTROL";
        TelnetOption[TelnetOption["LINEMODE"] = 34] = "LINEMODE";
        TelnetOption[TelnetOption["X_DISPLAY_LOCATION"] = 35] = "X_DISPLAY_LOCATION";
        TelnetOption[TelnetOption["ENVIRONMENT"] = 36] = "ENVIRONMENT";
        TelnetOption[TelnetOption["AUTHENTICATION"] = 37] = "AUTHENTICATION";
        TelnetOption[TelnetOption["ENCRYPTION"] = 38] = "ENCRYPTION";
        TelnetOption[TelnetOption["NEW_ENVIRONMENT"] = 39] = "NEW_ENVIRONMENT";
        TelnetOption[TelnetOption["TN3270E"] = 40] = "TN3270E";
        TelnetOption[TelnetOption["XAUTH"] = 41] = "XAUTH";
        TelnetOption[TelnetOption["CHARSET"] = 42] = "CHARSET";
        TelnetOption[TelnetOption["TELNET_REMOTE_SERIAL_PORT"] = 43] = "TELNET_REMOTE_SERIAL_PORT";
        TelnetOption[TelnetOption["COM_PORT_CONTROL"] = 44] = "COM_PORT_CONTROL";
        TelnetOption[TelnetOption["TELNET_SUPPRESS_LOCAL_ECHO"] = 45] = "TELNET_SUPPRESS_LOCAL_ECHO";
        TelnetOption[TelnetOption["TELNET_START_TLS"] = 46] = "TELNET_START_TLS";
        TelnetOption[TelnetOption["KERMIT"] = 47] = "KERMIT";
        TelnetOption[TelnetOption["SEND_URL"] = 48] = "SEND_URL";
        TelnetOption[TelnetOption["FORWARD_X"] = 49] = "FORWARD_X";
        TelnetOption[TelnetOption["TELOPT_PRAGMA_LOGON"] = 138] = "TELOPT_PRAGMA_LOGON";
        TelnetOption[TelnetOption["TELOPT_SSPI_LOGON"] = 139] = "TELOPT_SSPI_LOGON";
        TelnetOption[TelnetOption["TELOPT_PRAGMA_HEARTBEAT"] = 140] = "TELOPT_PRAGMA_HEARTBEAT";
        /** Generic MUD Communication Protocol option.
         * @example
         * [IAC, SB, GMCP, "Package.SubPackage", "JSON", IAC, SE]
         */
        TelnetOption[TelnetOption["GMCP"] = 201] = "GMCP";
        TelnetOption[TelnetOption["EXTENDED_OPTIONS_LIST"] = 255] = "EXTENDED_OPTIONS_LIST";
    })(TelnetOption = Telnet.TelnetOption || (Telnet.TelnetOption = {}));
    let TelnetOptionState;
    (function (TelnetOptionState) {
        TelnetOptionState[TelnetOptionState["DISABLED"] = 0] = "DISABLED";
        TelnetOptionState[TelnetOptionState["WAITING"] = 1] = "WAITING";
        TelnetOptionState[TelnetOptionState["ENABLED"] = 2] = "ENABLED";
    })(TelnetOptionState = Telnet.TelnetOptionState || (Telnet.TelnetOptionState = {}));
    class TelnetOptionMatrix {
        constructor() {
            this._options = {};
        }
        GetState(option) {
            if (this._options[option] === undefined) {
                this._options[option] = TelnetOptionState.DISABLED;
            }
            return this._options[option];
        }
        HasOption(option) {
            return this._options[option] !== undefined && this._options[option] === Telnet.TelnetOptionState.ENABLED;
        }
        SetState(option, state) {
            this._options[option] = state;
        }
    }
    Telnet.TelnetOptionMatrix = TelnetOptionMatrix;
    function ParseSequence(data) {
        // IAC COMMAND OPTION
        // IAC SB OPTION DATA IAC SE
        if (data.length >= 2 && data[0] === TelnetNegotiation.IAC && data[1] !== TelnetNegotiation.IAC) {
            const option = data[2];
            switch (data[1]) {
                // Command
                case TelnetNegotiation.SB:
                    // Subnegotiation
                    const offset = data.lastIndexOf(TelnetNegotiation.IAC);
                    let buf;
                    if (offset !== -1 && data[offset + 1] === TelnetNegotiation.SE) {
                        buf = data.slice(3, data.length - 2);
                        return {
                            option,
                            command: data[1],
                            data: buf,
                        };
                    }
                    else {
                        return null;
                    }
                    break;
                default:
                    return {
                        option,
                        command: data[1],
                    };
                    break;
            }
        }
        return null;
    }
    Telnet.ParseSequence = ParseSequence;
})(Telnet || (Telnet = {}));
module.exports = Telnet;
